@@ -71,7 +71,7 @@ verification.
     >>> ...
     >>> f = open("public_key.pem", "r")
     >>> hash_obj = SHA256.new(message)
-    >>> pub_key = DSA.importKey(f.read())
+    >>> pub_key = DSA.import_key(f.read())
     >>> if pub_key.verify(hash_obj, signature):
     >>>     print "OK"
     >>> else:
@@ -83,7 +83,7 @@ verification.
 """
 
 __all__ = ['generate', 'construct', 'DSAImplementation',
-           'DsaKey', 'importKey' ]
+           'DsaKey', 'import_key' ]
 
 import binascii
 import struct
@@ -97,14 +97,16 @@ from Crypto.Hash import SHA256
 from Crypto.Util.asn1 import (
                 DerObject, DerSequence,
                 DerInteger, DerObjectId,
-                DerBitString, newDerSequence,
-                newDerBitString
+                DerBitString,
                 )
 
 from Crypto.Math.Numbers import Integer
 from Crypto.Math.Primality import (test_probable_prime, COMPOSITE,
                                    PROBABLY_PRIME)
 
+from Crypto.PublicKey import (_expand_subject_public_key_info,
+                              _create_subject_public_key_info,
+                              _extract_subject_public_key_info)
 
 #   ; The following ASN.1 types are relevant for DSA
 #
@@ -137,7 +139,9 @@ from Crypto.Math.Primality import (test_probable_prime, COMPOSITE,
 class DsaKey(object):
     """Class defining an actual DSA key.
 
-    :undocumented: __getstate__, __setstate__, __repr__, __getattr__
+    :undocumented: __getstate__, __setstate__, __repr__, __getattr__,
+                   __init__, __eq__, __ne__, sign, verify, encrypt, decrypt,
+                   blind, unblind, size
     """
     #: Dictionary of DSA parameters.
     #:
@@ -330,7 +334,7 @@ class DsaKey(object):
 
         # DER format is always used, even in case of PEM, which simply
         # encodes it into BASE64.
-        params = newDerSequence(self.p, self.q, self.g)
+        params = DerSequence([self.p, self.q, self.g])
         if self.has_private():
             if pkcs8 is None:
                 pkcs8 = True
@@ -352,15 +356,14 @@ class DsaKey(object):
                 if format != 'PEM' and passphrase:
                     raise ValueError("DSA private key cannot be encrypted")
                 ints = [0, self.p, self.q, self.g, self.y, self.x]
-                binary_key = newDerSequence(*ints).encode()
+                binary_key = DerSequence(ints).encode()
                 key_type = "DSA PRIVATE"
         else:
             if pkcs8:
                 raise ValueError("PKCS#8 is only meaningful for private keys")
-            binary_key = newDerSequence(
-                            newDerSequence(DerObjectId(oid), params),
-                            newDerBitString(DerInteger(self.y))
-                            ).encode()
+
+            binary_key = _create_subject_public_key_info(oid,
+                                DerInteger(self.y), params)
             key_type = "DSA PUBLIC"
 
         if format == 'DER':
@@ -373,6 +376,29 @@ class DsaKey(object):
             return tobytes(pem_str)
         raise ValueError("Unknown key format '%s'. Cannot export the DSA key." % format)
 
+    # Methods defined in PyCrypto that we don't support anymore
+
+    def sign(self, M, K):
+        raise NotImplementedError("Use module Crypto.Signature.DSS instead")
+
+    def verify(self, M, signature):
+        raise NotImplementedError("Use module Crypto.Signature.DSS instead")
+
+    def encrypt(self, plaintext, K):
+        raise NotImplementedError
+
+    def decrypt(self, ciphertext):
+        raise NotImplementedError
+
+    def blind(self, M, B):
+        raise NotImplementedError
+
+    def unblind(self, M, B):
+        raise NotImplementedError
+
+    def size():
+        raise NotImplementedError
+
 
 def _generate_domain(L, randfunc):
     """Generate a new set of DSA domain parameters"""
@@ -461,6 +487,17 @@ def generate(bits, randfunc=None, domain=None):
 
     if domain:
         p, q, g = map(Integer, domain)
+
+        ## Perform consistency check on domain parameters
+        # P and Q must be prime
+        fmt_error = test_probable_prime(p) == COMPOSITE
+        fmt_error = test_probable_prime(q) == COMPOSITE
+        # Verify Lagrange's theorem for sub-group
+        fmt_error |= ((p - 1) % q) != 0
+        fmt_error |= g <= 1 or g >= p
+        fmt_error |= pow(g, q, p) != 1
+        if fmt_error:
+            raise ValueError("Invalid DSA domain parameters")
     else:
         p, q, g, _ = _generate_domain(bits, randfunc)
 
@@ -515,8 +552,9 @@ def construct(tup, consistency_check=True):
 
     fmt_error = False
     if consistency_check:
-        # Modulus must be prime
+        # P and Q must be prime
         fmt_error = test_probable_prime(key.p) == COMPOSITE
+        fmt_error = test_probable_prime(key.q) == COMPOSITE
         # Verify Lagrange's theorem for sub-group
         fmt_error |= ((key.p - 1) % key.q) != 0
         fmt_error |= key.g <= 1 or key.g >= key.p
@@ -532,70 +570,74 @@ def construct(tup, consistency_check=True):
 
     return key
 
-def _importKeyDER(key_data, passphrase, params):
+
+# Dss-Parms  ::=  SEQUENCE  {
+#       p       OCTET STRING,
+#       q       OCTET STRING,
+#       g       OCTET STRING
+# }
+# DSAPublicKey ::= INTEGER --  public key, y
+
+def _import_openssl_private(encoded, passphrase, params):
+    if params:
+        raise ValueError("DSA private key already comes with parameters")
+    der = DerSequence().decode(encoded, nr_elements=6, only_ints_expected=True)
+    if der[0] != 0:
+        raise ValueError("No version found")
+    tup = [der[comp] for comp in (4, 3, 1, 2, 5)]
+    return construct(tup)
+
+
+def _import_subjectPublicKeyInfo(encoded, passphrase, params):
+
+    algoid, encoded_key, emb_params =  _expand_subject_public_key_info(encoded)
+    if algoid != oid:
+        raise ValueError("No DSA subjectPublicKeyInfo")
+    if params and emb_params:
+        raise ValueError("Too many DSA parameters")
+
+    y = DerInteger().decode(encoded_key).value
+    p, q, g = list(DerSequence().decode(params or emb_params))
+    tup = (y, g, p, q)
+    return construct(tup)
+
+
+def _import_x509_cert(encoded, passphrase, params):
+
+    sp_info = _extract_subject_public_key_info(encoded)
+    return _import_subjectPublicKeyInfo(sp_info, None, params)
+
+
+def _import_pkcs8(encoded, passphrase, params):
+    if params:
+        raise ValueError("PKCS#8 already includes parameters")
+    k = PKCS8.unwrap(encoded, passphrase)
+    if k[0] != oid:
+        raise ValueError("No PKCS#8 encoded DSA key")
+    x = DerInteger().decode(k[1]).value
+    p, q, g = list(DerSequence().decode(k[2]))
+    tup = (pow(g, x, p), g, p, q, x)
+    return construct(tup)
+
+
+def _import_key_der(key_data, passphrase, params):
     """Import a DSA key (public or private half), encoded in DER form."""
 
-    try:
-        #
-        # Dss-Parms  ::=  SEQUENCE  {
-        #       p       OCTET STRING,
-        #       q       OCTET STRING,
-        #       g       OCTET STRING
-        # }
-        #
+    decodings = (_import_openssl_private,
+                 _import_subjectPublicKeyInfo,
+                 _import_x509_cert,
+                 _import_pkcs8)
 
-        # Try a simple private key first
-        if params:
-            x = DerInteger().decode(key_data).value
-            p, q, g = list(DerSequence().decode(params))    # Dss-Parms
-            tup = (pow(g, x, p), g, p, q, x)
-            return construct(tup)
-
-        der = DerSequence().decode(key_data)
-
-        # Try OpenSSL format for private keys
-        if len(der) == 6 and der.hasOnlyInts() and der[0] == 0:
-            tup = [der[comp] for comp in (4, 3, 1, 2, 5)]
-            return construct(tup)
-
-        # Try SubjectPublicKeyInfo
-        if len(der) == 2:
-            try:
-                algo = DerSequence().decode(der[0])
-                algo_oid = DerObjectId().decode(algo[0]).value
-                params = DerSequence().decode(algo[1])  # Dss-Parms
-
-                if algo_oid == oid and len(params) == 3 and\
-                        params.hasOnlyInts():
-                    bitmap = DerBitString().decode(der[1])
-                    pub_key = DerInteger().decode(bitmap.value)
-                    tup = [pub_key.value]
-                    tup += [params[comp] for comp in (2, 0, 1)]
-                    return construct(tup)
-            except (ValueError, EOFError):
-                pass
-
-        # Try to see if this is an X.509 DER certificate
-        # (Certificate ASN.1 type)
-        if len(der) == 3:
-            from Crypto.PublicKey import _extract_sp_info
-            try:
-                sp_info = _extract_sp_info(der)
-                return _importKeyDER(sp_info, passphrase, None)
-            except ValueError:
-                pass
-
-        # Try unencrypted PKCS#8
-        p8_pair = PKCS8.unwrap(key_data, passphrase)
-        if p8_pair[0] == oid:
-            return _importKeyDER(p8_pair[1], passphrase, p8_pair[2])
-
-    except (ValueError, EOFError):
-        pass
+    for decoding in decodings:
+        try:
+            return decoding(key_data, passphrase, params)
+        except ValueError:
+            pass
 
     raise ValueError("DSA key format is not supported")
 
-def importKey(extern_key, passphrase=None):
+
+def import_key(extern_key, passphrase=None):
     """Import a DSA key (public or private).
 
     :Parameters:
@@ -643,7 +685,7 @@ def importKey(extern_key, passphrase=None):
         (der, marker, enc_flag) = PEM.decode(tostr(extern_key), passphrase)
         if enc_flag:
             passphrase = None
-        return _importKeyDER(der, passphrase, None)
+        return _import_key_der(der, passphrase, None)
 
     if extern_key.startswith(b('ssh-dss ')):
         # This is probably a public OpenSSH key
@@ -659,10 +701,14 @@ def importKey(extern_key, passphrase=None):
 
     if bord(extern_key[0]) == 0x30:
         # This is probably a DER encoded key
-        return _importKeyDER(extern_key, passphrase, None)
+        return _import_key_der(extern_key, passphrase, None)
 
     raise ValueError("DSA key format is not supported")
 
+
+# Backward compatibility
+importKey = import_key
+
 #: `Object ID`_ for a DSA key.
 #:
 #: id-dsa ID ::= { iso(1) member-body(2) us(840) x9-57(10040) x9cm(4) 1 }
