@@ -82,17 +82,16 @@ them from known components. ElGamal keys allows you to perform basic signing,
 verification, encryption, and decryption.
 
     >>> from Crypto import Random
-    >>> from Crypto.Random import random
     >>> from Crypto.PublicKey import ElGamal
-    >>> from Crypto.Util.number import GCD
     >>> from Crypto.Hash import SHA
+    >>> from Crypto.Math import Numbers
     >>>
     >>> message = "Hello"
     >>> key = ElGamal.generate(1024, Random.new().read)
     >>> h = SHA.new(message).digest()
     >>> while 1:
-    >>>     k = random.StrongRandom().randint(1,key.p-1)
-    >>>     if GCD(k,key.p-1)==1: break
+    >>>     k = Numbers.random_range(min_inclusive=1, min_exclusive=key.p-1)
+    >>>     if k.gcd(key.p-1)==1: break
     >>> sig = key.sign(h,k)
     >>> ...
     >>> if key.verify(h,sig):
@@ -105,19 +104,15 @@ verification, encryption, and decryption.
 .. _ECRYPT: http://www.ecrypt.eu.org/documents/D.SPA.17.pdf
 """
 
-__all__ = ['generate', 'construct', 'ElGamalobj']
+__all__ = ['generate', 'construct', 'ElGamalKey']
 
-from Crypto.Util import number
 from Crypto import Random
-from Crypto.Util.number import (
-                GCD, isPrime,
-                getRandomRange, inverse,
-                size, getPrime
-                )
-
+from Crypto.Math.Primality import ( generate_probable_safe_prime,
+                                    test_probable_prime, COMPOSITE )
+from Crypto.Math.Numbers import Integer
 
 # Generate an ElGamal key with N bits
-def generate(bits, randfunc, progress_func=None):
+def generate(bits, randfunc):
     """Randomly generate a fresh, new ElGamal key.
 
     The key will be safe for use for both encryption and signature
@@ -131,39 +126,31 @@ def generate(bits, randfunc, progress_func=None):
             Random number generation function; it should accept
             a single integer N and return a string of random data
             N bytes long.
-        progress_func : callable
-            Optional function that will be called with a short string
-            containing the key parameter currently being generated;
-            it's useful for interactive applications where a user is
-            waiting for a key to be generated.
 
     :attention: You should always use a cryptographically secure random number generator,
         such as the one defined in the ``Crypto.Random`` module; **don't** just use the
         current time and the ``random`` module.
 
-    :Return: An ElGamal key object (`ElGamalobj`).
+    :Return: An ElGamal key object (`ElGamalKey`).
     """
-    obj=ElGamalobj()
+    obj=ElGamalKey()
     # Generate a safe prime p
     # See Algorithm 4.86 in Handbook of Applied Cryptography
-    if progress_func:
-        progress_func('p\n')
-    while 1:
-        q = getPrime(bits-1, randfunc)
-        obj.p = 2*q+1
-        if isPrime(obj.p, randfunc=randfunc):
-            break
+
+    obj.p = generate_probable_safe_prime(exact_bits=bits, randfunc=randfunc)
+    q = (obj.p - 1) >> 1
+
     # Generate generator g
     # See Algorithm 4.80 in Handbook of Applied Cryptography
     # Note that the order of the group is n=p-1=2q, where q is prime
-    if progress_func:
-        progress_func('g\n')
     while 1:
         # We must avoid g=2 because of Bleichenbacher's attack described
         # in "Generating ElGamal signatures without knowning the secret key",
         # 1996
         #
-        obj.g = getRandomRange(3, obj.p, randfunc)
+        obj.g = Integer.random_range(min_inclusive=3,
+                                     max_exclusive=obj.p,
+                                     randfunc=randfunc)
         safe = 1
         if pow(obj.g, 2, obj.p)==1:
             safe=0
@@ -176,18 +163,16 @@ def generate(bits, randfunc, progress_func=None):
         # g^{-1} must not divide p-1 because of Khadir's attack
         # described in "Conditions of the generator for forging ElGamal
         # signature", 2011
-        ginv = inverse(obj.g, obj.p)
+        ginv = obj.g.inverse(obj.p)
         if safe and (obj.p-1) % ginv == 0:
             safe=0
         if safe:
             break
     # Generate private key x
-    if progress_func:
-        progress_func('x\n')
-    obj.x=getRandomRange(2, obj.p-1, randfunc)
+    obj.x = Integer.random_range(min_inclusive=2,
+                                 max_exclusive=obj.p-1,
+                                 randfunc=randfunc)
     # Generate public key y
-    if progress_func:
-        progress_func('y\n')
     obj.y = pow(obj.g, obj.x, obj.p)
     return obj
 
@@ -215,17 +200,17 @@ def construct(tup):
 
     :Raise PublicKey.ValueError:
         When the key being imported fails the most basic ElGamal validity checks.
-    :Return: An ElGamal key object (`ElGamalobj`).
+    :Return: An ElGamal key object (`ElGamalKey`).
     """
 
-    obj=ElGamalobj()
+    obj=ElGamalKey()
     if len(tup) not in [3,4]:
         raise ValueError('argument for construct() wrong length')
     for i in range(len(tup)):
         field = obj._keydata[i]
-        setattr(obj, field, tup[i])
+        setattr(obj, field, Integer(tup[i]))
 
-    fmt_error = not isPrime(obj.p)
+    fmt_error = test_probable_prime(obj.p) == COMPOSITE
     fmt_error |= obj.g<=1 or obj.g>=obj.p
     fmt_error |= pow(obj.g, obj.p-1, obj.p)!=1
     fmt_error |= obj.y<1 or obj.y>=obj.p
@@ -238,7 +223,7 @@ def construct(tup):
 
     return obj
 
-class ElGamalobj(object):
+class ElGamalKey(object):
     """Class defining an ElGamal key.
 
     :undocumented: __getstate__, __setstate__, __repr__, __getattr__
@@ -264,32 +249,36 @@ class ElGamalobj(object):
 
     def _encrypt(self, M, K):
         a=pow(self.g, K, self.p)
-        b=( M*pow(self.y, K, self.p) ) % self.p
-        return ( a,b )
+        b=( pow(self.y, K, self.p)*M ) % self.p
+        return map(int, ( a,b ))
 
     def _decrypt(self, M):
         if (not hasattr(self, 'x')):
             raise TypeError('Private key not available in this object')
-        r = getRandomRange(2, self.p-1, self._randfunc)
-        a_blind = (M[0] * pow(self.g, r, self.p)) % self.p
+        r = Integer.random_range(min_inclusive=2,
+                                 max_exclusive=self.p-1,
+                                 randfunc=self._randfunc)
+        a_blind = (pow(self.g, r, self.p) * M[0]) % self.p
         ax=pow(a_blind, self.x, self.p)
-        plaintext_blind = (M[1] * inverse(ax, self.p ) ) % self.p
+        plaintext_blind = (ax.inverse(self.p) * M[1] ) % self.p
         plaintext = (plaintext_blind * pow(self.y, r, self.p)) % self.p
-        return plaintext
+        return int(plaintext)
 
     def _sign(self, M, K):
         if (not hasattr(self, 'x')):
             raise TypeError('Private key not available in this object')
         p1=self.p-1
-        if (GCD(K, p1)!=1):
+        K = Integer(K)
+        if (K.gcd(p1)!=1):
             raise ValueError('Bad K value: GCD(K,p-1)!=1')
         a=pow(self.g, K, self.p)
-        t=(M-self.x*a) % p1
+        t=(Integer(M)-self.x*a) % p1
         while t<0: t=t+p1
-        b=(t*inverse(K, p1)) % p1
-        return (a, b)
+        b=(t*K.inverse(p1)) % p1
+        return map(int, (a, b))
 
     def _verify(self, M, sig):
+        sig = map(Integer, sig)
         if sig[0]<1 or sig[0]>self.p-1:
             return 0
         v1=pow(self.y, sig[0], self.p)
@@ -299,15 +288,18 @@ class ElGamalobj(object):
             return 1
         return 0
 
-    def size(self):
-        return size(self.p) - 1
-
     def has_private(self):
         if hasattr(self, 'x'):
             return 1
         else:
             return 0
 
+    def can_encrypt(self):
+        return True
+
+    def can_sign(self):
+        return True
+
     def publickey(self):
         return construct((self.p, self.g, self.y))
 
@@ -329,5 +321,25 @@ class ElGamalobj(object):
         from pickle import PicklingError
         raise PicklingError
 
+    # Methods defined in PyCrypto that we don't support anymore
 
-object=ElGamalobj
+    def sign(self, M, K):
+        raise NotImplementedError
+
+    def verify(self, M, signature):
+        raise NotImplementedError
+
+    def encrypt(self, plaintext, K):
+        raise NotImplementedError
+
+    def decrypt(self, ciphertext):
+        raise NotImplementedError
+
+    def blind(self, M, B):
+        raise NotImplementedError
+
+    def unblind(self, M, B):
+        raise NotImplementedError
+
+    def size():
+        raise NotImplementedError
